"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDeSerializeFunctions = exports.createHandleUndefinedForRequiredValueMethod = exports.createErrorMethod = exports.THROW_NULLABLE_ERROR_METHOD_NAME = exports.REQUIRE_NOT_NULLABLE_METHOD_NAME = void 0;
const utils_1 = require("./utils");
exports.REQUIRE_NOT_NULLABLE_METHOD_NAME = 'requireNotNullable';
exports.THROW_NULLABLE_ERROR_METHOD_NAME = 'throwNullableError';
const createErrorMethod = (models) => {
    models.addFunction({
        name: exports.THROW_NULLABLE_ERROR_METHOD_NAME,
        statements: 'throw new Error("expected not nullable value, but was null");',
        returnType: 'void',
        isExported: true,
        parameters: []
    });
};
exports.createErrorMethod = createErrorMethod;
const createHandleUndefinedForRequiredValueMethod = (models) => {
    models.addFunction({
        name: exports.REQUIRE_NOT_NULLABLE_METHOD_NAME,
        statements: `\
            if (value !== null && value !== undefined) { return value; }
            if (process.env.NODE_ENV === 'development') { ${exports.THROW_NULLABLE_ERROR_METHOD_NAME}(); }
            return defaultValue;`,
        returnType: 'any',
        isExported: true,
        parameters: [
            { name: 'value', type: 'any' },
            { name: 'defaultValue', type: 'any | undefined' }
        ]
    });
};
exports.createHandleUndefinedForRequiredValueMethod = createHandleUndefinedForRequiredValueMethod;
const createDeSerializeFunctions = (models) => {
    (0, exports.createErrorMethod)(models);
    (0, exports.createHandleUndefinedForRequiredValueMethod)(models);
    forEnum(models);
    forTypeAliases(models);
    forInterfaces(models);
};
exports.createDeSerializeFunctions = createDeSerializeFunctions;
const forEnum = (models) => {
    models.getEnums().forEach((e) => {
        models.addFunctions([{
                isExported: true,
                name: e.getName() + "Serialize",
                statements: `return value`,
                returnType: 'string',
                parameters: [{ name: 'value', type: e.getName(), }]
            }, {
                isExported: true,
                name: e.getName() + "Deserialize",
                statements: `return value as ${e.getName()}`,
                returnType: e.getName(),
                parameters: [{ name: 'value', type: 'string', }]
            }]);
    });
};
const forTypeAliases = (models) => {
    models.getTypeAliases().forEach((alias) => {
        var _a, _b;
        models.addFunctions([{
                name: alias.getName() + "Serialize",
                isExported: true,
                statements: (_a = alias.getTypeNode()) === null || _a === void 0 ? void 0 : _a.getText().split('|').map(x => x.trim()).map(x => `if (value.$type === '${x}') return ${x}Serialize(value);`).join('\n'),
                returnType: 'any',
                parameters: [{ name: 'value', type: alias.getName(), }]
            }, {
                isExported: true,
                name: alias.getName() + "Deserialize",
                statements: ((_b = alias.getTypeNode()) === null || _b === void 0 ? void 0 : _b.getText().split('|').map(x => x.trim()).map(x => `if (value.$type === '${x}') return ${x}Deserialize(value);`).join('\n')) + `\nthrow new Error('Unknown type ' + value.$type);`,
                returnType: alias.getName(),
                parameters: [{ name: 'value', type: 'any', }]
            },]);
    });
};
const forInterfaces = (models) => {
    models.getInterfaces().forEach((inter) => {
        var _a, _b, _c, _d, _e, _f;
        const indexSignatures = (_a = inter === null || inter === void 0 ? void 0 : inter.getIndexSignatures()) !== null && _a !== void 0 ? _a : [];
        if (indexSignatures.length > 1) {
            console.error(`Cannot handle interface with multiple index signatures ${inter.getName()}. Ignoring.`);
            return;
        }
        if (indexSignatures.length === 1) {
            const [first] = indexSignatures;
            const deserializeIndexValue = (0, utils_1.property)(models, (_c = (_b = first.getReturnTypeNode()) === null || _b === void 0 ? void 0 : _b.getText()) !== null && _c !== void 0 ? _c : 'unknown', false, 'innerValue');
            const serializeIndexValue = (0, utils_1.property)(models, (_e = (_d = first.getReturnTypeNode()) === null || _d === void 0 ? void 0 : _d.getText()) !== null && _e !== void 0 ? _e : 'unknown', true, 'innerValue');
            models.addFunctions([{
                    name: inter.getName() + "Serialize",
                    statements: `return Object.entries(value).reduce((all, [key, innerValue]: [string, any]) => ({...all, [key]: ${serializeIndexValue}}), {})`,
                    returnType: 'any',
                    isExported: true,
                    parameters: [{ name: 'value', type: inter.getName(), }]
                }, {
                    name: inter.getName() + "Deserialize",
                    isExported: true,
                    statements: `return Object.entries(value).reduce((all, [key, innerValue]: [string, any]) => ({...all, [key]: ${deserializeIndexValue}}), {})`,
                    returnType: inter.getName(),
                    parameters: [{ name: 'value', type: 'any', }]
                }]);
            return;
        }
        const extend = (_f = inter.getExtends()[0]) === null || _f === void 0 ? void 0 : _f.getText();
        const seriPropSet = (inter.getProperties().map((prop) => `result['${prop.getName()}'] = ${(0, utils_1.property)(models, prop, true, `value['${prop.getName()}']`)};`)).join('\n');
        const desiPropSet = (inter.getProperties().map((prop) => `result['${prop.getName()}'] = ${(0, utils_1.property)(models, prop, false, `value['${prop.getName()}']`)};`)).join('\n');
        models.addFunctions([{
                name: inter.getName() + "Serialize",
                statements: `const result: any = ` + (extend ? `${extend}Serialize(value);` : `{};`) + `\n${seriPropSet}\nreturn result;`,
                returnType: 'object',
                isExported: true,
                parameters: [{ name: 'value', type: inter.getName(), }]
            }, {
                name: inter.getName() + "Deserialize",
                isExported: true,
                statements: `const result: any = ` + (extend ? `${extend}Deserialize(value);` : `{};`) + `\n${desiPropSet}\nreturn result as any;`,
                returnType: inter.getName(),
                parameters: [{ name: 'value', type: 'any', }]
            }]);
    });
};
//# sourceMappingURL=serializefunc.js.map